"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const event = require('codeceptjs/lib/event');
class GenerateCtrfReport {
    constructor(reporterOptions) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        this.reporterName = 'jest-ctrf-json-reporter';
        this.defaultOutputFile = 'ctrf-report.json';
        this.defaultOutputDir = 'ctrf';
        this.filename = this.defaultOutputFile;
        this.attachListeners();
        this.reporterConfigOptions = {
            outputFile: (_a = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.outputFile) !== null && _a !== void 0 ? _a : this.defaultOutputFile,
            outputDir: (_b = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.outputDir) !== null && _b !== void 0 ? _b : this.defaultOutputDir,
            appName: (_c = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.appName) !== null && _c !== void 0 ? _c : undefined,
            appVersion: (_d = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.appVersion) !== null && _d !== void 0 ? _d : undefined,
            osPlatform: (_e = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.osPlatform) !== null && _e !== void 0 ? _e : undefined,
            osRelease: (_f = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.osRelease) !== null && _f !== void 0 ? _f : undefined,
            osVersion: (_g = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.osVersion) !== null && _g !== void 0 ? _g : undefined,
            buildName: (_h = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.buildName) !== null && _h !== void 0 ? _h : undefined,
            buildNumber: (_j = reporterOptions === null || reporterOptions === void 0 ? void 0 : reporterOptions.buildNumber) !== null && _j !== void 0 ? _j : undefined,
        };
        this.ctrfReport = {
            results: {
                tool: {
                    name: 'codeceptjs',
                },
                summary: {
                    tests: 0,
                    passed: 0,
                    failed: 0,
                    pending: 0,
                    skipped: 0,
                    other: 0,
                    start: 0,
                    stop: 0,
                },
                tests: [],
            },
        };
        this.ctrfEnvironment = {};
        if (((_k = this.reporterConfigOptions) === null || _k === void 0 ? void 0 : _k.outputFile) !== undefined)
            this.setFilename(this.reporterConfigOptions.outputFile);
        if (!fs.existsSync((_l = this.reporterConfigOptions.outputDir) !== null && _l !== void 0 ? _l : this.defaultOutputDir)) {
            fs.mkdirSync((_m = this.reporterConfigOptions.outputDir) !== null && _m !== void 0 ? _m : this.defaultOutputDir, { recursive: true });
        }
    }
    attachListeners() {
        event.dispatcher.on(event.all.before, () => {
            var _a;
            this.ctrfReport.results.summary.start = Date.now();
            this.setEnvironmentDetails((_a = this.reporterConfigOptions) !== null && _a !== void 0 ? _a : {});
            if (this.hasEnvironmentDetails(this.ctrfEnvironment)) {
                this.ctrfReport.results.environment = this.ctrfEnvironment;
            }
        });
        event.dispatcher.on(event.test.passed, (test) => {
            this.addTestResult(test, 'passed');
        });
        event.dispatcher.on(event.test.failed, (test) => {
            this.addTestResult(test, 'failed');
        });
        event.dispatcher.on(event.test.skipped, (test) => {
            this.addTestResult(test, 'skipped');
        });
        event.dispatcher.on(event.all.result, () => {
            this.ctrfReport.results.summary.stop = Date.now();
            this.writeReportToFile(this.ctrfReport);
        });
    }
    setFilename(filename) {
        if (filename.endsWith('.json')) {
            this.filename = filename;
        }
        else {
            this.filename = `${filename}.json`;
        }
    }
    calculateTestDuration(test) {
        let duration = 0;
        if (Array.isArray(test.steps)) {
            test.steps.forEach((step) => {
                duration += step.duration;
            });
        }
        return duration;
    }
    addTestResult(test, status) {
        const testDuration = this.calculateTestDuration(test);
        this.ctrfReport.results.tests.push({
            name: test.title,
            status,
            duration: testDuration !== null && testDuration !== void 0 ? testDuration : 0,
        });
        this.ctrfReport.results.summary[status]++;
        this.ctrfReport.results.summary.tests++;
    }
    setEnvironmentDetails(reporterConfigOptions) {
        if (reporterConfigOptions.appName !== undefined) {
            this.ctrfEnvironment.appName = reporterConfigOptions.appName;
        }
        if (reporterConfigOptions.appVersion !== undefined) {
            this.ctrfEnvironment.appVersion = reporterConfigOptions.appVersion;
        }
        if (reporterConfigOptions.osPlatform !== undefined) {
            this.ctrfEnvironment.osPlatform = reporterConfigOptions.osPlatform;
        }
        if (reporterConfigOptions.osRelease !== undefined) {
            this.ctrfEnvironment.osRelease = reporterConfigOptions.osRelease;
        }
        if (reporterConfigOptions.osVersion !== undefined) {
            this.ctrfEnvironment.osVersion = reporterConfigOptions.osVersion;
        }
        if (reporterConfigOptions.buildName !== undefined) {
            this.ctrfEnvironment.buildName = reporterConfigOptions.buildName;
        }
        if (reporterConfigOptions.buildNumber !== undefined) {
            this.ctrfEnvironment.buildNumber = reporterConfigOptions.buildNumber;
        }
    }
    hasEnvironmentDetails(environment) {
        return Object.keys(environment).length > 0;
    }
    writeReportToFile(data) {
        var _a;
        const filePath = path.join((_a = this.reporterConfigOptions.outputDir) !== null && _a !== void 0 ? _a : this.defaultOutputDir, this.filename);
        const str = JSON.stringify(data, null, 2);
        try {
            fs.writeFileSync(filePath, str + '\n');
            console.log(`${this.reporterName}: successfully written ctrf json to %s/%s`, this.reporterConfigOptions.outputDir, this.filename);
        }
        catch (error) {
            console.error(`Error writing ctrf json report:, ${String(error)}`);
        }
    }
}
exports.default = GenerateCtrfReport;
